 package cs.cinvestav;
 
 import java.io.PrintStream;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Comparator;
 import java.util.HashMap;
 import java.util.LinkedHashMap;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
 
 public class GoIsingAPI
 {
   public static int COLOR_BLANCO = 1;
   public static int COLOR_NEGRO = 2;
   public static int COLOR_NEUTRO = 0;
   private int[][] tablero;
   private int tamanioTablero;
   
   public GoIsingAPI(int tamanioTablero)
   {
     this.tamanioTablero = tamanioTablero;
     this.tablero = new int[tamanioTablero][tamanioTablero];
   }
   
   /*public static <K, V extends Comparable<? super V>> Map<K, V> sortByValue(Map<K, V> map)
   {
     List<Map.Entry<K, V>> list = new LinkedList(map.entrySet());
     
     Collections.sort(list, new Comparator()
     {
       public int compare(Map.Entry<K, V> o1, Map.Entry<K, V> o2)
       {
         return ((Comparable)o1.getValue()).compareTo(o2.getValue());
       }
     });
     Map<K, V> result = new LinkedHashMap();
     for (Map.Entry<K, V> entry : list) {
       result.put(entry.getKey(), entry.getValue());
     }
     return result;
   }*/
   
   static class ValueComparator implements Comparator<Map.Entry >{
       @Override
       public int compare(Map.Entry o1, Map.Entry o2)
       {
         return (((Comparable)o1.getValue()).compareTo(o2.getValue()));
       }
   }
   
   public static <K, V extends Comparable<? super V>> Map<K, V> sortByValue(Map<K, V> map)
   {
     List<Map.Entry<K, V>> list = new LinkedList(map.entrySet());
     ValueComparator comp = new ValueComparator();
     Collections.sort(list, comp);
     /* Collections.sort(list, new Comparator()
     {

       public int compare(Map.Entry<K, V> o1, Map.Entry<K, V> o2)
       {
         return ((Comparable)o1.getValue()).compareTo(o2.getValue());
       }
     });
     */
     Map<K, V> result = new LinkedHashMap();
     for (Map.Entry<K, V> entry : list) {
       result.put(entry.getKey(), entry.getValue());
     }
     return result;
   }
   
   private double procesarMejoresJugadas(int colorJugador)
   {
     int[][] nuevoTablero = new int[this.tamanioTablero][this.tamanioTablero];
     for (int x = 0; x < this.tamanioTablero; x++) {
       for (int y = 0; y < this.tamanioTablero; y++)
       {
         int jugador = getFicha(y, x);
         if (jugador == COLOR_BLANCO) {
           nuevoTablero[x][y] = ProcesadorTablero.ETIQUETA_BLANCO;
         } else if (jugador == COLOR_NEGRO) {
           nuevoTablero[x][y] = ProcesadorTablero.ETIQUETA_NEGRO;
         } else if (jugador == COLOR_NEUTRO) {
           nuevoTablero[x][y] = ProcesadorTablero.ETIQUETA_ESPACIO;
         }
       }
     }
     ProcesadorTablero procesadorTablero = new ProcesadorTablero(nuevoTablero, this.tamanioTablero);
     
     procesadorTablero.leerGrafo();
     
 
     CFG cfg = CFG.generarCFG(procesadorTablero.getPiedrasCompuestas(), procesadorTablero.getLibertadesEspeciales());
     
 
     int tamanio = cfg.getNodos().size();
     
     double[][] matrizW = new double[tamanio][tamanio];
     double[] vectorX = new double[tamanio];
     double[] vectorH = new double[tamanio];
     for (int x = 0; x < tamanio; x++) {
       for (int y = 0; y < tamanio; y++)
       {
         double valor = 0.0D;
         if (x != y) {
           for (AristaCFG arista : ((NodoCFG)cfg.getNodos().get(x)).getAristas()) {
             if (arista.getDestino().getId() == y) {
               valor += arista.getPeso();
             }
           }
         }
         matrizW[x][y] = valor;
       }
     }
     for (int x = 0; x < tamanio; x++)
     {
       NodoCFG nodo = (NodoCFG)cfg.getNodos().get(x);
       double valor = nodo.getLibertades();
       vectorH[x] = valor;
     }
     for (int x = 0; x < tamanio; x++)
     {
       NodoCFG nodo = (NodoCFG)cfg.getNodos().get(x);
       double valor = (nodo.getTamanio() + Math.pow(CFG.VALOR_OJO, nodo.getOjos()) - 1.0D) * nodo.getColor();
       vectorX[x] = valor;
     }
     double valorNegras = 0.0D;
     double valorBlancas = 0.0D;
     for (int x = 0; x < tamanio; x++)
     {
       double valor = 0.0D;
       NodoCFG nodo = (NodoCFG)cfg.getNodos().get(x);
       for (int y = 0; y < tamanio; y++) {
         valor += -0.5D * (vectorX[x] * vectorX[y] * matrizW[x][y]);
       }
       valor -= vectorH[x] * Math.abs(vectorX[x]);
       if (nodo.getColor() == -1) {
         valorNegras += valor;
       } else {
         valorBlancas += valor;
       }
     }
     if (colorJugador == COLOR_BLANCO) {
       return valorBlancas;
     }
     return valorNegras;
   }
   
   public void setFicha(int color, int posicionX, int posicionY)
   {
     try
     {
       this.tablero[posicionX][posicionY] = color;
     }
     catch (Exception e)
     {
       System.err.println("Imposible colocar una ficha... (" + posicionX + "," + posicionY + ") fuera de los limites del tablero");
     }
   }
   
   public int getFicha(int posicionX, int posicionY)
   {
     int color;
     try
     {
       color = this.tablero[posicionX][posicionY];
     }
     catch (Exception e)
     {
       color = -1;
       System.err.println("Imposible obtener una ficha... (" + posicionX + "," + posicionY + ") fuera de los limites del tablero");
     }
     return color;
   }
   
   public MejoresJugadas obtenerMejoresJugadas(int color)
   {
     Map<Punto, Double> mapBlancas = new HashMap();
     Map<Punto, Double> mapNegras = new HashMap();
     for (int x = 0; x < this.tamanioTablero; x++) {
       for (int y = 0; y < this.tamanioTablero; y++) {
         if (getFicha(x, y) == COLOR_NEUTRO)
         {
           setFicha(color, x, y);
           double valorBlancas = procesarMejoresJugadas(COLOR_BLANCO);
           mapBlancas.put(new Punto(x, y), Double.valueOf(valorBlancas));
           double valorNegras = procesarMejoresJugadas(COLOR_NEGRO);
           mapNegras.put(new Punto(x, y), Double.valueOf(valorNegras));
           
           setFicha(COLOR_NEUTRO, x, y);
         }
       }
     }
     mapBlancas = sortByValue(mapBlancas);
     mapNegras = sortByValue(mapNegras);
     
     System.out.println("Blancas tirando blanca: " + mapBlancas);
     System.out.println("Negras tirando blanca: " + mapNegras);
     
     return new MejoresJugadas(mapBlancas, mapNegras);
   }
 }



/* Location:           C:\Users\Arturo\Documents\ProyectoIsing\ProyectoIsing\GoIsingAPI - copia\lib\GoIsingLibrary\

 * Qualified Name:     cs.cinvestav.GoIsingAPI

 * JD-Core Version:    0.7.0.1

 */